package nccloud.zjnytwo_cd.zjnytwo_cd.bankcredithvo.action;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import nc.bs.dao.BaseDAO;
import nc.bs.dao.DAOException;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.md.model.MetaDataException;
import nc.md.persist.framework.IMDPersistenceQueryService;
import nc.vo.pub.BusinessException;
import nc.vo.pub.lang.UFDate;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditBVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditBillVO;
import nc.vo.zjnytwo_cd.bankcredit.BankcreditHVO;
import nccloud.framework.core.exception.ExceptionUtils;
import nccloud.framework.core.io.WebFile;
import nccloud.framework.web.action.itf.ICommonAction;
import nccloud.framework.web.container.IRequest;
import nccloud.pubitf.riart.pflow.CloudPFlowContext;
import nccloud.pubitf.riart.pflow.ICloudScriptPFlowService;
import nccloud.vo.zjnytwo_cd.zjnytwo_cd.BankcreditHVOConst;
import nccloud.zjnytwo_cd.zjnytwo_cd.bankcredithvo.zjnytwo_cd.util.CommonUtil;
import nccloud.zjnytwo_cd.zjnytwo_cd.tool.Excel;

/**
 * 银行授信填报导入
 * @author LP
 *
 */
public class BankcreditHVOImportAction implements ICommonAction {
	
	public String group;
	public String org;
	public String org_v;
	public int month;
	
	public BaseDAO dao;
	public IMDPersistenceQueryService qryService;

	@Override
	public Object doAction(IRequest paramIRequest) {
		
		WebFile[] readFiles = paramIRequest.readFiles();
		
		WebFile file = readFiles[0];

		try {
			
			dao = new BaseDAO();
			qryService = NCLocator.getInstance().lookup(IMDPersistenceQueryService.class);
			
			JSONArray jsonArray = Excel.readXLSX(file);//解析Excel
			
			BankcreditBillVO bankcreditBillVO = setBankcreditBillVO(jsonArray);
			
			saveBankcreditBillVO(bankcreditBillVO);
			
		} catch (Exception e) {
			Logger.error(e);
		    ExceptionUtils.wrapException(e);
		}
		
		return null;
	}
	
	/*
	 * 保存VO
	 */
	private void saveBankcreditBillVO(BankcreditBillVO bankcreditBillVO) throws BusinessException {
		
		String oldbankcredit = SqlQuery("pk_bankcredit", "zjnytwo_cd_bankcredit_h", "month", String.valueOf(month));
		
		if (oldbankcredit!=null) {
			BankcreditBillVO oldBankareditBillVO = getVO(oldbankcredit);
			deleteVO(oldBankareditBillVO);//删除VO
		}
		saveVO(bankcreditBillVO);
	}
	
	/*
	 * 组合BankcreditBillVO
	 */
	private BankcreditBillVO setBankcreditBillVO(JSONArray jsonArray) throws Exception {
		
		BankcreditBillVO bankcreditBillVO = new BankcreditBillVO();
		
		JSONArray headArray = jsonArray.getJSONArray(0);//表头
		JSONArray bodyArray = jsonArray.getJSONArray(1);//表体
		
		BankcreditHVO headHvo = sethead(headArray);
		BankcreditBVO[] bodyBvos = setbody(bodyArray);
		
		bankcreditBillVO.setParentVO(headHvo);
		bankcreditBillVO.setChildrenVO(bodyBvos);
		
		return bankcreditBillVO;
	}
	
	/*
	 * 导入表头
	 */
	private BankcreditHVO sethead(JSONArray headArray) throws Exception {
		
		BankcreditHVO bankcreditHVO = new BankcreditHVO();
		JSONObject headObject = headArray.getJSONObject(0);
		
		String enablestate = headObject.getString("启用");
		
		if (headArray.size()>1) {
			throw new Exception("表头不允许有多列！请检查数据的合法性或Excel表格是否符合规范！");
		}
		
		if (enablestate==null) {
			throw new Exception("启用为必填项，请检查！");
		}
		
		if(enablestate.equals("Y")) {
			bankcreditHVO.setEnablestate(2);
		}else if(enablestate.equals("N")) {
			bankcreditHVO.setEnablestate(3);
		}else {
			throw new Exception("启用只允许填写'N'和'Y'，请检查！");
		}
		
		group = SqlQuery("pk_group", "org_group", "code", headObject.getString("集团"));
		org = SqlQuery("pk_org", "org_orgs", "code", headObject.getString("组织"));
		org_v = SqlQuery("pk_vid", "org_orgs_v", "code", headObject.getString("组织"));;
		
		bankcreditHVO.setPk_group(group);
		bankcreditHVO.setPk_org(org);
		bankcreditHVO.setPk_org_v(headObject.getString("组织"));
		
		month = Integer.valueOf(headObject.getString("月份"));
		bankcreditHVO.setMonth(month);//月份
		
		Date date = new Date();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd"); 
		String Inputdate = formatter.format(date);
		bankcreditHVO.setInputdate(new UFDate(Inputdate));//导入日期
		bankcreditHVO.setDbilldate(new UFDate(Inputdate));//单据日期
		
		bankcreditHVO.setInputman(SqlQuery("cuserid", "sm_user", "user_code", headObject.getString("导入人")));
		bankcreditHVO.setCode(headObject.getString("编码"));
		bankcreditHVO.setName(headObject.getString("名称"));
		bankcreditHVO.setVnote(headObject.getString("备注"));
		
		return bankcreditHVO;
	}
	
	/*
	 * 导入表体
	 */
	private BankcreditBVO[] setbody(JSONArray bodyArray) throws DAOException {
		
		BankcreditBVO[] bvos = new BankcreditBVO[bodyArray.size()];
		
		for(int i = 0 ; i < bodyArray.size() ; i++ ) {
			
			JSONObject bodyobject = bodyArray.getJSONObject(i);
			
			BankcreditBVO bankcreditBVO = new BankcreditBVO();
			
			bankcreditBVO.setPk_group(group);
			bankcreditBVO.setPk_org(org);
			bankcreditBVO.setPk_org_v(org_v);
			bankcreditBVO.setCrowno(bodyobject.getString("行号"));
			bankcreditBVO.setUnit(SqlQuery("pk_org", "org_orgs", "code", bodyobject.getString("一级单位")));
			bankcreditBVO.setCreditunit(SqlQuery("pk_org", "org_orgs", "code", bodyobject.getString("授信单位")));
			bankcreditBVO.setBanktype_code(bodyobject.getString("银行类别编码"));
			bankcreditBVO.setBanktype(bodyobject.getString("银行类别"));
			bankcreditBVO.setCurrency(bodyobject.getString("币种"));
			bankcreditBVO.setAmount(Integer.valueOf(bodyobject.getString("有效授信金额")));
			bankcreditBVO.setUsedamount(Integer.valueOf(bodyobject.getString("已使用金额")));
			bankcreditBVO.setRemainderamount(Integer.valueOf(bodyobject.getString("有效剩余金额")));
			bankcreditBVO.setAmountloan(Integer.valueOf(bodyobject.getString("已使用金额_贷款")));
			bankcreditBVO.setAmountcredit(Integer.valueOf(bodyobject.getString("已使用金额_信用证")));
			bankcreditBVO.setAmountdraft(Integer.valueOf(bodyobject.getString("已使用金额_承兑汇票")));
			bankcreditBVO.setVbnote(bodyobject.getString("行备注"));
			
			bvos[i] = bankcreditBVO;
		}
		
		return bvos;
	}
	
	/*
	 * VO查询方法
	 */
	@SuppressWarnings("unchecked")
	private BankcreditBillVO getVO(String pk_bankcredit) throws MetaDataException {
		ArrayList<BankcreditBillVO> AggVOs = (ArrayList<BankcreditBillVO>) qryService.queryBillOfVOByCond(BankcreditBillVO.class,"pk_bankcredit = '"+pk_bankcredit+"' and dr = '0'",true,false);
		
		BankcreditBillVO bankcreditBillVO = AggVOs.get(0);
		
		return bankcreditBillVO;
	}
	
	/*
	 * SQL查询方法
	 */
	private String SqlQuery(String out,String table,String lock,String key) throws DAOException {
		
		if (key==null) {
			return null;
		}
		
		String sql = "select "+out+" from "+table+" where "+lock+" = '"+key+"' and dr = '0'";
		
		Object object = dao.executeQuery(sql, new ColumnProcessor());
		
		if (lock.equals("month") && object == null) {
			return null;
		}else if (object == null) {
			throw new DAOException("根据值"+key+"在数据库中未查到相应的数据，请检查！");
		}
		
		return object.toString();
	}
	
	/*
	 * 保存脚本
	 */
	@SuppressWarnings("unchecked")
	private BankcreditBillVO saveVO(BankcreditBillVO... aggVOs)
		      throws BusinessException {

		    String actionCode = BankcreditHVOConst.CONST_ACTION_SAVEBASE;
		    String billType = BankcreditHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service =
		        CommonUtil.getCloudScriptPFlowService();

		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug("调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]结束");

		    String wheresql =
		        "pk_bankcredit" + "='"
		            + ((BankcreditBillVO) result[0]).getPrimaryKey() + "'";
		    Collection<BankcreditBillVO> bills =
		        CommonUtil.getMDPersistenceQueryService().queryBillOfVOByCond(
		            BankcreditBillVO.class, wheresql, true, false);
		    return bills.toArray(new BankcreditBillVO[0])[0];
	}
	
	/*
	 * 删除脚本
	 */
	private Object deleteVO(BankcreditBillVO... aggVOs)
		      throws BusinessException {

		    if ((aggVOs == null) || (aggVOs.length == 0)) {
		      return null;
		    }

		    String actionCode = BankcreditHVOConst.CONST_ACTION_DELETE;
		    String billType = BankcreditHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service = CommonUtil.getCloudScriptPFlowService();
		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug(
		        "调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType + "]结束");

		    return result;
	}

}
