package nccloud.zjnytwo_cd.zjnytwo_cd.bankloanhvo.action;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import nc.bs.dao.BaseDAO;
import nc.bs.dao.DAOException;
import nc.bs.framework.common.NCLocator;
import nc.bs.logging.Logger;
import nc.jdbc.framework.processor.ColumnProcessor;
import nc.md.model.MetaDataException;
import nc.md.persist.framework.IMDPersistenceQueryService;
import nc.vo.pub.BusinessException;
import nc.vo.pub.lang.UFDate;
import nc.vo.zjnytwo_cd.bankloan.BankloanBVO;
import nc.vo.zjnytwo_cd.bankloan.BankloanBillVO;
import nc.vo.zjnytwo_cd.bankloan.BankloanHVO;
import nccloud.framework.core.exception.ExceptionUtils;
import nccloud.framework.core.io.WebFile;
import nccloud.framework.web.action.itf.ICommonAction;
import nccloud.framework.web.container.IRequest;
import nccloud.pubitf.riart.pflow.CloudPFlowContext;
import nccloud.pubitf.riart.pflow.ICloudScriptPFlowService;
import nccloud.vo.zjnytwo_cd.zjnytwo_cd.BankloanHVOConst;
import nccloud.zjnytwo_cd.zjnytwo_cd.bankloanhvo.zjnytwo_cd.util.CommonUtil;
import nccloud.zjnytwo_cd.zjnytwo_cd.tool.Excel;

/**
 * 银行贷款填报导入
 * @author LP
 *
 */
public class BankloanHVOImportAction implements ICommonAction {

	public String group;
	public String org;
	public String org_v;
	public int month;
	
	public BaseDAO dao;
	public IMDPersistenceQueryService qryService;

	@Override
	public Object doAction(IRequest paramIRequest) {
		
		WebFile[] readFiles = paramIRequest.readFiles();
		
		WebFile file = readFiles[0];

		try {
			
			dao = new BaseDAO();
			qryService = NCLocator.getInstance().lookup(IMDPersistenceQueryService.class);
			
			JSONArray jsonArray = Excel.readXLSX(file);//解析Excel
			
			BankloanBillVO bankloanBillVO = setBankloanBillVO(jsonArray);
			
			saveBankloanBillVO(bankloanBillVO);
			
		} catch (Exception e) {
			Logger.error(e);
		    ExceptionUtils.wrapException(e);
		}
		
		return null;
	}
	
	/*
	 * 保存VO
	 */
	private void saveBankloanBillVO(BankloanBillVO bankloanBillVO) throws BusinessException {
		
		String oldbankloan = SqlQuery("pk_bankloan", "zjnytwo_cd_bankloan_h", "month", String.valueOf(month));
		
		if (oldbankloan!=null) {
			BankloanBillVO oldBankloanBillVO = getVO(oldbankloan);
			deleteVO(oldBankloanBillVO);//删除VO
		}
		saveVO(bankloanBillVO);
	}
	
	/*
	 * 组合BankcreditBillVO
	 */
	private BankloanBillVO setBankloanBillVO(JSONArray jsonArray) throws Exception {
		
		BankloanBillVO bankloanBillVO = new BankloanBillVO();
		
		JSONArray headArray = jsonArray.getJSONArray(0);//表头
		JSONArray bodyArray = jsonArray.getJSONArray(1);//表体
		
		BankloanHVO headHvo = sethead(headArray);
		BankloanBVO[] setbody = setbody(bodyArray);
		
		bankloanBillVO.setParentVO(headHvo);
		bankloanBillVO.setChildrenVO(setbody);
		
		return bankloanBillVO;
	}
	
	/*
	 * 导入表头
	 */
	private BankloanHVO sethead(JSONArray headArray) throws Exception {
		
		BankloanHVO bankloantHVO = new BankloanHVO();
		JSONObject headObject = headArray.getJSONObject(0);
		
		String enablestate = headObject.getString("启用");
		
		if (headArray.size()>1) {
			throw new Exception("表头不允许有多列！请检查数据的合法性或Excel表格是否符合规范！");
		}
		
		if (enablestate==null) {
			throw new Exception("启用为必填项，请检查！");
		}
		
		if(enablestate.equals("Y")) {
			bankloantHVO.setEnablestate(2);
		}else if(enablestate.equals("N")) {
			bankloantHVO.setEnablestate(3);
		}else {
			throw new Exception("启用只允许填写'N'和'Y'，请检查！");
		}
		
		group = SqlQuery("pk_group", "org_group", "code", headObject.getString("集团"));
		org = SqlQuery("pk_org", "org_orgs", "code", headObject.getString("组织"));
		org_v = SqlQuery("pk_vid", "org_orgs_v", "code", headObject.getString("组织"));;
		
		bankloantHVO.setPk_group(group);
		bankloantHVO.setPk_org(org);
		bankloantHVO.setPk_org_v(headObject.getString("组织"));
		
		month = Integer.valueOf(headObject.getString("月份"));
		bankloantHVO.setMonth(month);//月份
		
		Date date = new Date();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd"); 
		String Inputdate = formatter.format(date);
		bankloantHVO.setInputdate(new UFDate(Inputdate));//导入日期
		bankloantHVO.setDbilldate(new UFDate(Inputdate));//单据日期
		
		bankloantHVO.setInputman(SqlQuery("cuserid", "sm_user", "user_code", headObject.getString("导入人")));
		bankloantHVO.setCode(headObject.getString("编码"));
		bankloantHVO.setName(headObject.getString("名称"));
		bankloantHVO.setVnote(headObject.getString("备注"));
		
		return bankloantHVO;
	}
	
	/*
	 * 导入表体
	 */
	private BankloanBVO[] setbody(JSONArray bodyArray) throws DAOException {
		
		BankloanBVO[] bvos = new BankloanBVO[bodyArray.size()];
		
		for(int i = 0 ; i < bodyArray.size() ; i++ ) {
			
			JSONObject bodyobject = bodyArray.getJSONObject(i);
			
			BankloanBVO bankloanBVO = new BankloanBVO();
			
			bankloanBVO.setPk_group(group);
			bankloanBVO.setPk_org(org);
			bankloanBVO.setPk_org_v(org_v);
			bankloanBVO.setCrowno(bodyobject.getString("行号"));
			bankloanBVO.setUnit(SqlQuery("pk_org", "org_orgs", "code", bodyobject.getString("一级单位")));
			bankloanBVO.setUseunit(SqlQuery("pk_org", "org_orgs", "code", bodyobject.getString("使用单位")));
			bankloanBVO.setBankname(bodyobject.getString("银行名称"));
			bankloanBVO.setLoanamount(Integer.valueOf(bodyobject.getString("借款金额")));
			bankloanBVO.setRepayamount(Integer.valueOf(bodyobject.getString("已还款金额")));
			bankloanBVO.setLoanbalance(Integer.valueOf(bodyobject.getString("借款余额")));
			bankloanBVO.setLoantype(bodyobject.getString("借款种类"));
			bankloanBVO.setStartdate(new UFDate(bodyobject.getString("借款起始日")));
			bankloanBVO.setEnddate(new UFDate(bodyobject.getString("借款到期日")));
			bankloanBVO.setTerm(bodyobject.getString("期限"));
			bankloanBVO.setRates(bodyobject.getString("实际利率"));
			bankloanBVO.setSettletype(bodyobject.getString("解决类型"));
			bankloanBVO.setBackedunit(bodyobject.getString("担保单位"));
			bankloanBVO.setDocumentcode(bodyobject.getString("单据编号"));
			bankloanBVO.setVbnote(bodyobject.getString("行备注"));
			
			bvos[i] = bankloanBVO;
		}
		
		return bvos;
	}
	
	/*
	 * VO查询方法
	 */
	@SuppressWarnings("unchecked")
	private BankloanBillVO getVO(String pk_bankcredit) throws MetaDataException {
		ArrayList<BankloanBillVO> AggVOs = (ArrayList<BankloanBillVO>) qryService.queryBillOfVOByCond(BankloanBillVO.class,"pk_bankloan = '"+pk_bankcredit+"' and dr = '0'",true,false);
		
		BankloanBillVO bankloanBillVO = AggVOs.get(0);
		
		return bankloanBillVO;
	}
	
	/*
	 * SQL查询方法
	 */
	private String SqlQuery(String out,String table,String lock,String key) throws DAOException {
		
		if (key==null) {
			return null;
		}
		
		String sql = "select "+out+" from "+table+" where "+lock+" = '"+key+"' and dr = '0'";
		
		Object object = dao.executeQuery(sql, new ColumnProcessor());
		
		if (lock.equals("month") && object == null) {
			return null;
		}else if (object == null) {
			throw new DAOException("根据值"+key+"在数据库中未查到相应的数据，请检查！");
		}
		
		return object.toString();
	}
	
	/*
	 * 保存脚本
	 */
	@SuppressWarnings("unchecked")
	private BankloanBillVO saveVO(BankloanBillVO... aggVOs)
		      throws BusinessException {

		    String actionCode = BankloanHVOConst.CONST_ACTION_SAVEBASE;
		    String billType = BankloanHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service =
		        CommonUtil.getCloudScriptPFlowService();

		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug("调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]结束");

		    String wheresql =
		        "pk_bankloan" + "='"
		            + ((BankloanBillVO) result[0]).getPrimaryKey() + "'";
		    Collection<BankloanBillVO> bills =
		        CommonUtil.getMDPersistenceQueryService().queryBillOfVOByCond(
		            BankloanBillVO.class, wheresql, true, false);
		    return bills.toArray(new BankloanBillVO[0])[0];
	}
	
	/*
	 * 删除脚本
	 */
	private Object deleteVO(BankloanBillVO... aggVOs)
		      throws BusinessException {

		    if ((aggVOs == null) || (aggVOs.length == 0)) {
		      return null;
		    }

		    String actionCode = BankloanHVOConst.CONST_ACTION_DELETE;
		    String billType = BankloanHVOConst.CONST_BILLTYPE_COST;

		    CloudPFlowContext context = new CloudPFlowContext();
		    context.setActionName(actionCode);
		    context.setBillType(billType);
		    context.setBillVos(aggVOs);
		    Logger.debug("开始调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType
		        + "]...");

		    ICloudScriptPFlowService service = CommonUtil.getCloudScriptPFlowService();
		    Object[] result = service.exeScriptPFlow(context);

		    Logger.debug(
		        "调用动作脚本 ActionName[" + actionCode + "] BillType[" + billType + "]结束");

		    return result;
		  }

}
